module ActiveSchema
  class Column

    def initialize connection, table_name, column_name, column_type, column_opts
      @connection, @table_name, @column_name, @column_type, @column_opts = \
        connection, table_name, column_name, column_type, column_opts
      return add_column unless column_exists?
      update_column_type_if_needed
      update_column_if_needed
    end

    private
    def update_column_type_if_needed
      update_column(@column_type) unless column_exists?(@column_type)
    end

    def update_column_if_needed
      @connection.reset! # reconnect to get fresh columns info
      @column_opts.each_pair do |o,v|
        next if column_exists?(@column_type, o => v)
        update_column(@column_type, o => v)
      end
    end

    def update_column type, opts = {}
      @connection.change_column(@table_name, @column_name, type, opts)
    end

    def column_exists? type = nil, opts = {}
      @connection.column_exists?(@table_name, @column_name, type, opts)
    end

    def add_column
      @connection.add_column(@table_name, @column_name, @column_type, @column_opts)
    end

    class Pruner
      def initialize connection, table_name, specified_columns
        @connection, @table_name, @specified_columns = \
          connection, table_name, specified_columns.map(&:to_s)
        @connection.reset!
        prune_as_needed
      end

      private
      def prune_as_needed
        (existing_columns - Array(primary_key) - @specified_columns).each do |column_name|
          remove_column(column_name)
        end
      end

      def primary_key
        @connection.primary_key(@table_name)
      end

      def remove_column column_name
        @connection.remove_column(@table_name, column_name)
      end

      def existing_columns
        @connection.columns(@table_name).map(&:name).map(&:to_s)
      end
    end
  end
end
